
cd "I:\bokhari\fb_research\analysis\adhd\training\data"

use simulateddrugs01, clear

keep fips year s4-s7 so4-so7 lpo1-lpo17 lpoz1-lpoz17    ///
           lpoi2 lpoii2  lpzi2 lpzii2 y2 t1 t2          ///
           lnpop lnkids lnmds lncaiddrugs lnmcaidenrollees pcpi  ///
           msac stabr cenregc cenregn cendivc cendivn cntyst poptot  ///
	       msapmsa99c  ///
		   r4-r7 po4-po7 poz1-poz17
		   
drop if year < 2000

rename  y2 		y   	
rename  lpoi2 	lpi  
rename 	lpoii2	lpii
rename	lpzi2	lpzi
rename	lpzii2  lpzii

rename  so4  s01	
rename  so5  s02
rename  so6  s03
rename  so7  s04	

rename  s4   s1
rename 	s5   s2
rename  s6   s3
rename  s7   s4

rename  r4   r1
rename	r5   r2
rename  r6   r3
rename 	r7   r4

rename  po4  p1
rename 	po5  p2
rename  po6  p3
rename	po7  p4

gen lp1 =  lpo4
gen lp2 = lpo5
gen lp3 = lpo6
gen lp4 = lpo7

gen lpz1 = lpoz4
gen  lpz2 = lpoz5
gen  lpz3 = lpoz6
gen  lpz4 = lpoz7

gen ly = ln(y)
gen lypi = ly - lpi
gen lypzi = ly - lpzi
gen t1ly = t1*ly
gen t2ly = t2*ly

global eq1  "(s1 lypi t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees lp1 lp2 lp3 lp4)"
global eq2  "(s2 lypi t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees lp1 lp2 lp3 lp4)"
global eq3  "(s3 lypi t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees lp1 lp2 lp3 lp4)"
global eq4  "(s4 lypi t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees lp1 lp2 lp3 lp4)"
global enlist "(lp1 lp2 lp3 lp4 lypi)"
global exlist "(lpz1 lpz2 lpz3 lpz4 lpzi ly t1ly t2ly)"


/*symmetry*/
constraint 1 [s1]lp2 = [s2]lp1
constraint 2 [s1]lp3 = [s3]lp1
constraint 3 [s2]lp3 = [s3]lp2   

/* Homogenity restrictions  */
constraint 4 [s1]lp1 + [s1]lp2 + [s1]lp3 + [s1]lp4 = 0
constraint 5 [s2]lp1 + [s2]lp2 + [s2]lp3 + [s2]lp4 = 0
constraint 6 [s3]lp1 + [s3]lp2 + [s3]lp3 + [s3]lp4 = 0 
 

reg3 $eq1 $eq2 $eq3, endog($enlist) exog($exlist) constr(1 2 3 4 5 6)  3sls


/* aidsills is canned routine that also computes own and cross elasticities 
 --- but implements expenidture in a different way 
 Specifically, it does not allow for fixed shares specified by the user 
 in the computation of the price index and so in the term beta(ln(R/P)) 
 it either uses actual shares or shares from the overall sample ... sbar .. 
 rather than from the first period as we do in the computation of lpi 
 above. 
 
 Furher, since they compute elasticities at the mean, then s-bar cancels out 
 in the computation of elasticity when it is multiplied by beta. Thus, they would
 use the simpler formula e_ij = mu_ij/s_i - delat_ij  whereu_ij is as described 
 in their paper. 
 
 */
 
/* (ssc?) install aidsills if you don't have it */

aidsills s1 s2 s3 s4 , prices (p1 p2 p3 p4) exp(y) ///
         intercept(t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees) ///
		 ivprices(lpz1 lpz2 lpz3 lpz4) ///
		 ivexpenditure(lpzi) /// 
		 homogeneity symmetry

/*post estimation elasticitiy command */		 
aidsills_elas

