
ods html newfile = proc;

libname training  "I:\bokhari\fb_research\analysis\adhd\training\data";
data logits;
  set training.simulateddrugs02;
  drop kids;  /* the variable already exits as kids5t19 */
  poi = poi/100;
  pzi = pzi/100;
run; quit;

proc contents data = logits;
run; quit;

/*add dummies for brands */
%include  "I:\bokhari\fb_research\analysis\contrib\contrib_progs\dummy.sas";
%dummy(data=logits, out=foo, var =drug , prefix = drug, name=, fullrank=0) ;

data foo2;
/****************
outside good, potential market and shares:

Potential market is defined via 30mg * 3 of mph per day.
 a ADHD kid gets 30mg per day.
 --> 30/1000 grams per day.
 --> 30*30/1000 grams per month.
--> (30*30*12)/1000 grams per year.
--> .9*12 grams per year. 
--> 10.8 grams per year. 

mso -> potential market 15% of all children + 5% of other adults
consuming at 3 times the DDD amount  OR 3 times the current consumption
rate per child. 

*****************/
set foo;
  qpc = qq/(.07*.75*kids5t19);    ** quantity per child;
  mso = ((kids5t19)*.15 + (poptot - (kids5t19 + kids0t4))*.05)*(3*qpc);
  q0 = mso - qq ;                 ** outside good;
  s0 = q0/mso;                    ** share of outside good;
  lns0 =   log(s0);               ** log of share of outside good; 
  sqi = qi/mso;                   ** re-compute shares relative to potential market ;

 /* or use 
 qpc = qq/(.1*.60*kids5t19);    ** quantity per child;
 mso = ((kids5t19)*.1 + (poptot - (kids5t19 + kids0t4))*0.01)*(qpc);
  q0 = mso - qq ;                 ** outside good;
  s0 = q0/mso;                    ** share of outside good;
  lns0 =   log(s0);               ** log of share of outside good; 
  sqi = qi/mso;                   ** re-compute shares relative to potential market ;
*/

  if sqi ~ = 0 then lnsqi =   log(sqi);  ** log of shares;
  if sqi ~ = 0 then  lnsqim0 = lnsqi - lns0;
  if sqi = 0 then lnsqi = .; 

  if drug in ('01' '02' '03' '04' '05' '06' '07' '08') then mph = 1; else mph = 0;
  if drug in ('09' '10' '11') then mas = 1; else mas = 0;
  if drug in ('12' '13' '14' '15' '16' ) then dex = 1; else dex = 0;
  if drug in ('17') then oth = 1; else oth = 0;

 if drug in ('01' '02' '03' '09' '10' '12' '13' '14') then ir = 1; else ir = 0;
 if drug in ('04' '05' '06' '07' '15' '16') then er = 1; else er = 0;
 if drug in ('08' '11') then xr = 1; else xr = 0;


if year = 1999 then delete;

RUN; QUIT;


/*add dummies for states */
%include  "I:\bokhari\fb_research\analysis\contrib\contrib_progs\dummy.sas";
%dummy(data=foo2, out=foo2, var =stabr , prefix = d, name=, fullrank=0) ;

data logits;
  set foo2;
 run; quit;

proc datasets lib = work;
save logits;
run; quit;


proc means data = logits;
var sqi s0 poi;
run; quit;


proc reg data = logits;
  model lnsqim0 = poi mph mas dex ir er;
run; quit;


proc reg data = logits;
  model lnsqim0 = poi mph mas dex ir er
  t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees d1-d48 ; 
run; quit;


proc reg data = logits;
  model lnsqim0 = poi drug1-drug16 ;
run; quit;


ods graphics off;

proc syslin data = logits  2sls ;

ENDOGENOUS  lnsqim0 poi ;
INSTRUMENTS pzi t1 t2  
  lnkids lnmds lncaiddrugs lnmcaidenrollees
  drug1-drug16 d1-d48;

 model lnsqim0 = poi t1 t2  
  lnkids lnmds lncaiddrugs lnmcaidenrollees
  drug1-drug16 d1-d48;

 run; quit;




 
 proc means data = logits mean;
 var poi sqi;
 class drug;
 *where year = 2003;
 output out = means mean = ;
 run; quit;

data means (drop = _type_ _freq_);
  set means;
  if _type_ = 0 then delete;
run; quit;
  

 /*

 for elasticity formula see Nevo 2000 JEMS, p522
 note that p enters in his model as -p so elasticity formulas 
are reversed interms of multiplication with -1.

 
*/

proc iml;
  /*read in the stuff above and make elasticity matrix */

m1f1names = {'Ritalin','Methylin','Generics (MPH-IR)'};
m1f2names = {'Ritalin SRLA', 'Metadate ERCD', 'MethylinER','Generics (MPH-ER)'};
m1f3names = {'Concerta'};
m2f1names = {'Aderall','Generic (MAS-IR)'};
m2f2names = {'Aderall XR'};
m3f1names = {'Dexederine','Dextrostat','Generics (DEX-IR)'};
m3f2names = {'Dexederine SR','Generics (DEX-ER)'};
m4f1names = {'Offlabels'};
names1 = m1f1names//m1f2names//m1f3names//m2f1names//m2f2names//m3f1names//m3f2names//m4f1names;

use means;
read all var{drug} into drug;
read all var{poi} into pbar;
read all var{sqi} into sqbar;
close means;

print names1 drug pbar sqbar; 


biv = -5.21234;
bols = -0.62377; 
etaiv = j(17,17,0);
etaols = j(17,17,0);

do j = 1 to 17;
do k = 1 to 17;
  etaiv[j,k] = -biv*pbar[k]*sqbar[k];
  etaiv[j,j] = biv*pbar[j]*(1-sqbar[j]);

  etaols[j,k] = -bols*pbar[k]*sqbar[k];
  etaols[j,j] =  bols*pbar[j]*(1-sqbar[j]);

end;
end;



mattrib etaiv label = {'Logit price elasticities -- IV estimates [Entry (j,k) --> DlnQj/DlnPk]'}
format = 9.3
colname = names1 rowname = names1;


mattrib etaols label = {'Logit price elasticities -- OLS estimates [Entry (j,k) --> DlnQj/DlnPk]'}
format = 9.3
colname = names1 rowname = names1;


print etaiv;
print etaols;

quit;


/* try it on aggregate data */


 proc means data = logits noprint ;
 var poi sqi lnsqim0 mph mas dex ir er pzi ;
 class drug;
 *where year = 2003;
 output out = aggregate mean = ;
 run; quit;


 
data aggregate (drop = _type_ _freq_);
  set aggregate;
  if _type_ = 0 then delete;
run; quit;


proc reg data = aggregate ;
 model lnsqim0 = poi  mph mas dex ir er ;
run; quit;



proc syslin data = logits 2sls ;

ENDOGENOUS  lnsqim0 poi ;
INSTRUMENTS pzi mph mas dex ir er  ;

 model lnsqim0 = poi mph mas dex ir er ;

 run; quit;

