
options nodate center nonotes nonumber linesize = 100 formdlim='#';
%let progspath = I:\bokhari\fb_research\analysis\adhd\training\progs\;
libname training 'I:\bokhari\fb_research\analysis\adhd\training\data\';


/********************************************************************
  1. Create a temp dataset  work.aids2 from training.simulateddrugs01
  2. Create a temp dataset  work.demo with just demographic variables 
*********************************************************************/
data work.aids2;   
set training.simulateddrugs01;
run; quit;


data work.demo;
  set aids2 (keep = fips year t1 lnpop lnkids lnmds lncaiddrugs lnmcaidenrollees pcpi t2
           msac stabr cenregc cenregn cendivc cendivn cntyst poptot
	      msapmsa99c  d1-d49);

   t1lnkids = t1*lnkids;
   t2lnkids = t2*lnkids;
   t3lnkids = t3*lnkids;

   t1lnmds = t1*lnmds;
   t2lnmds = t2*lnmds;
   t3lnmds = t3*lnmds;

   t1lncaiddrugs = t1*lncaiddrugs;
   t2lncaiddrugs = t2*lncaiddrugs;
   t3lncaiddrugs = t3*lncaiddrugs;

   t1lnmcaidenrollees = t1*lnmcaidenrollees;
   t2lnmcaidenrollees = t2*lnmcaidenrollees;
   t3lnmcaidenrollees = t3*lnmcaidenrollees;


run; quit;



/**************************** DEFINE MACRO SETFIRSTYR *************************

Define the macro: SETFIRSTYR(arg#1,arg#2) where
   arg#1 --> Name of the input and an output data set
   arg#2 --> Value of the first year of the data

Purpose: Creates two datasets: (a) Same as the input dataset (arg#1) but
with all observations deleted that were less than the value the year
specified by arg#2 and (b) Additional data set named
"prefirstyr_arg#1" where this second dataset contains observations
from years before the value of arg#2.

Example: setfirstyr(segment101,2001) will read in the initial data set
work.segment101 and create two data sets (a) work.segment101 with
observations starting in 2001 and the second data set named
work.prefirstyr_segment101 with observations ending in 2000.

******************************************************************************/


%MACRO SETFIRSTYR(INDATA,YR);

  data prefirstyr_&indata;
  set &indata;
  if year < &yr then output;
  run; quit;

  data &indata;
  set &indata;
  if year < &yr then delete;
  run; quit;

%MEND;





/**************************** DEFINE MACRO PANELV1 ****************************

Define the macro: panelv1(arg#1,arg#2) where
   arg#1 --> Name of the input and the output dataset
   arg#2 --> Length of the panel measured in years

Purpose: The macro constructs a balanced panel. The input dataset has obs. at
the county-year level. While most (non-rural) counties have observations in
all years for the segment, some may have a year missing. In that case, this
macro deletes those counties completely. For the named dataset (arg#1), it
deletes all observations from a county, i.e., drops the county completely, if
the number of observations for a county is less than the length of the panel
(arg#2). Thus if county xyz has only three years of observations but the
panel is supposed to be 4 years long, then it drop the county xyz from the
dataset.

Example: panelv1(segment101,4). Reads in the dataset work.segment101
and drops observations from all counties that have less than 4 years
of data available in them.


Logic of the Macro:
  1. The macro reads in the input dataset
  2. It then counts the number of observations per county.
  3. It creates a new (temp) indicator variable (called  dropcounty) and sets
     it equal to 1 if the number of years for a county is less than the stated
     length (say 4).
  4. Finally it deletes all those counties marked for deletion.


******************************************************************************/


%MACRO PANELV1(INDATA,LENGTH);


    proc means data = &indata noprint;
    var year;
    class fips;
    output out = foo n=numyears;
    run; quit;

    data foo (drop = _type_ _freq_);
    set foo;
    if _type_ = 0 then delete;
    if numyears = &length then dropcounty = 0; else dropcounty = 1;
    label
    numyears = "How Many Years Data Available?"
    dropcounty = "Drop this county to obtain a balanced panel? 1 = Yes, Drop it."
    ;
    run; quit;

    proc sort data = foo; by fips; run; quit;
    proc sort data = &indata; by fips year; run; quit;

    data &indata (drop = numyears dropcounty);
    merge &indata (in=a) foo (in =b);
    by fips;
    if dropcounty = 0 then output;
    run; quit;

    proc sort data = &indata; by fips year; run; quit;

    proc datasets lib = work nolist;
    delete foo;
    run; quit;

%MEND;




/*********************************************************************
1. Create a smaller dataset named work.segment101 from work.aids2
   such that this dataset has sales figures etc. for all drugs 
   in Molecule 1, Form 1 (i.e. MPH-IR).
 

***********************************************************************/

data segment101;
   set aids2 (keep = fips year s1 s2 s3 lpo1-lpo17 lpoz1-lpoz17 lpoi1 lpoii1  lpzi1 lpzii1 y1 so1-so3 t1 t2 );
   rename 
   y1     = y           
   lpoi1  = lpi
   lpoii1 = lpii
   lpzi1  = lpzi
   lpzii1 = lpzii
   so1    = s01
   so2    = s02
   so3    = s03
   ;
   lp1   = lpo1;
   lp2   = lpo2;
   lp3   = lpo3;
   
   lpz1  = lpoz1;
   lpz2  = lpoz2;
   lpz3 =  lpoz3;

run; quit;   

  data segment101;
   set segment101;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
run; quit; 

proc sort data = segment101; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data segment101; merge segment101(in=a) demo(in=b);  by fips year; 
  t1ly = t1*ly;
  t2ly = t2*ly;
if a = 1 then output;
run; quit;
%setfirstyr(segment101,2000);                 /*Choose first year of the Panel */
%panelv1(segment101,4);                       /*Balance the panel */

data namesegment101;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = Ritalin TAB (Grp1)
Brand02 = Methylin TAB (Grp7)
Brand03 = Generic Meth TAB (Grp20)
;
run; quit;


/**************************************************************/

data segment122 ;
   set aids2 (keep = fips year s4-s7 so4-so7 lpo1-lpo17 lpoz1-lpoz17 lpoi2 lpoii2  lpzi2 lpzii2 y2 t1 t2  
       );
   rename 
   y2     = y           
   lpoi2  = lpi
   lpoii2 = lpii
   lpzi2  = lpzi
   lpzii2 = lpzii
   so4    = s01
   so5    = s02
   so6    = s03
   so7    = s04
   s4    = s1
   s5    = s2
   s6    = s3
   s7    = s4
   ;

    
   lp1   = lpo4;
   lp2   = lpo5;
   lp3   = lpo6;
   lp4   = lpo7;

   lpz1  = lpoz4;
   lpz2  = lpoz5;
   lpz3 =  lpoz6;
   lpz4 =  lpoz7;

run; quit;   

  data segment122;
   set segment122;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;

run; quit; 



proc sort data = segment122; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data segment122; merge segment122(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(segment122,2000);                 /*Choose first year of the Panel */
%panelv1(segment122,4);                       /*Balance the panel */

data namesegment122;
infile datalines dsd dlm='=';
input brand $ description $50.;
datalines ;
Brand01 = Ritalin SR,LA  (wGrp2+wGrp3)
Brand02 = Metadate ER,CD (wGrp3+wGrp5)
Brand04 = Methylin ER (Grp8)
Brand05 = Generic Meth SR(Grp21)
;
run; quit;



/**************************************************************/

data segment201;
   set aids2 (keep = fips year s9-s10 so9-so10 lpo1-lpo17 lpoz1-lpoz17 lpoi4 lpoii4  lpzi4 lpzii4 y4 t1 t2);
   rename 
   y4     = y           
   lpoi4  = lpi
   lpoii4 = lpii
   lpzi4  = lpzi
   lpzii4 = lpzii
   so9    = s01
   so10   = s02
   s9    = s1
   s10    = s2
   ;

   lp1   = lpo9;
   lp2   = lpo10;

   lpz1   = lpoz9;
   lpz2   = lpoz10;



run; quit;   

  data segment201;
   set segment201;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit; 

proc sort data = segment201; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data segment201; merge segment201(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(segment201,2002);                 /*Choose first year of the Panel */
%panelv1(segment201,2);                       /*Balance the panel */

data namesegment201;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = Adderall TAB (Grp10)
Brand02 = Generic Amphetamine Mixed Salts (Grp24)
;

run; quit;



/**************************************************************/

data segment300;
   set aids2 (keep = fips year s12-s14 so12-so14 lpo1-lpo17 lpoz1-lpoz17 lpoi6 lpoii6  lpzi6 lpzii6 y6 t1 t2);
   rename 
   y6     = y           
   lpoi6  = lpi
   lpoii6 = lpii
   lpzi6  = lpzi
   lpzii6 = lpzii
   so12   = s01
   so13   = s02
   so14   = s03
   s12    = s1
   s13    = s2
   s14    = s3
   ;

   lp1   = lpo12;
   lp2   = lpo13;
   lp3   = lpo14;

   lpz1   = lpoz12;
   lpz2   = lpoz13;
   lpz3   = lpoz14;


run; quit;   

  data segment300;
   set segment300;
   if y = 0 or y = . then delete;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit; 

proc sort data = segment300; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data segment300; merge segment300(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(segment300,2001);                 /*Choose first year of the Panel */
%panelv1(segment300,3);                       /*Balance the panel */



data namesegment300;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = Dexedrine TAB  (Grp14)
Brand02 = Dextrostat TAB (Grp13)
Brand03 = Generic Dextroamphetamine TAB (Grp26)
;

run; quit;




/**************************************************************/

data segment301;
   set aids2 (keep = fips year s15-s16 so15-so16 lpo1-lpo17 lpoz1-lpoz17 lpoi7 lpoii7  lpzi7 lpzii7 y7 t1 t2 );
   rename 
   y7     = y           
   lpoi7  = lpi
   lpoii7 = lpii
   lpzi7  = lpzi
   lpzii7 = lpzii
   so15   = s01
   so16   = s02
   s15    = s1
   s16    = s2
   ;

   lp1   = lpo15;
   lp2   = lpo16;

   lpz1   = lpoz15;
   lpz2   = lpoz16;

run; quit;   

  data segment301;
   set segment301;
   if y = 0 or y = . then delete;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit; 

proc sort data = segment301; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data segment301; merge segment301(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(segment301,2002);                 /*Choose first year of the Panel */
%panelv1(segment301,2);                       /*Balance the panel */


data namesegment301;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = Dexedrine SRCAP (Grp15)
Brand02 = Generic Dextroamphetamine SRCAP (Grp27)
;

run; quit;


/**************************************************************/

data methylphenidates  (drop = lpoii1-lpoii3 lpzii1-lpzii3 );
   set aids2 (keep = fips year u1-u3 uo1-uo3  lpoii1-lpoii3  lpzii1-lpzii3  lpomi1 lpomii1 lpzmi1 lpzmii1
                      yy1 t1 t2 lpo1-lpo17 lpoz1-lpoz17);
   rename 
   yy1     = y           
   lpomi1  = lpi
   lpomii1 = lpii
   lpzmi1  = lpzi
   lpzmii1 = lpzii
   uo1    = s01
   uo2    = s02
   uo3    = s03
   u1     = s1
   u2     = s2
   u3     = s3
   ;

   lp1   = lpoii1;
   lp2   = lpoii2;
   lp3   = lpoii3;

   lpz1  = lpzii1;
   lpz2  = lpzii2;
   lpz3  = lpzii3;

run; quit;   

  data methylphenidates;
   set methylphenidates;
   if y = 0 or y = . then delete;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;

   t1lpz1 = t1*lpz1;
   t1lpz2 = t1*lpz2;
   t1lpz3 = t1*lpz3;
   t1lpzi = t1*lpzi;

   t2lpz1 = t2*lpz1;
   t2lpz2 = t2*lpz2;
   t2lpz3 = t2*lpz3;
   t2lpzi = t2*lpzi;


   lpz1lpz2 = lpz1*lpz2;
   lpz1lpz3 = lpz1*lpz3;
   lpz1lpzi = lpz1*lpzi;
   lpz2lpz3 = lpz2*lpz3;
   lpz2lpzi = lpz2*lpzi;
   lpz3lpzi = lpz3*lpzi;

run; quit; 

proc sort data = methylphenidates; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data methylphenidates; merge methylphenidates(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(methylphenidates,2000);                 /*Choose first year of the Panel */
%panelv1(methylphenidates,4);                       /*Balance the panel */


data namemethylphenidates;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = MPH-IR
Brand02 = MPH-ER
Brand03 = MPH-OROS
;

run; quit;




/**************************************************************/

data amphatamines  (drop = lpoii4-lpoii5 lpzii4-lpzii5);
   set aids2 (keep = fips year u4-u5 uo4-uo5  lpoii4-lpoii5  lpzii4-lpzii5  lpomi2 lpomii2 lpzmi2 lpzmii2
                      yy2 t1 t2 lpo1-lpo17 lpoz1-lpoz17);
   rename 
   yy2     = y           
   lpomi2  = lpi
   lpomii2 = lpii
   lpzmi2  = lpzi
   lpzmii2 = lpzii
   uo4    = s01
   uo5    = s02
   u4     = s1
   u5     = s2
   ;

   lp1   = lpoii4;
   lp2   = lpoii5;

   lpz1  = lpzii4;
   lpz2  = lpzii5;

run; quit;   

  data amphatamines;
   set amphatamines;
   if y = 0 or y = . then delete;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit; 

proc sort data = amphatamines; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data amphatamines; merge amphatamines(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(amphatamines,2001);                 /*Choose first year of the Panel */
%panelv1(amphatamines,3);                       /*Balance the panel */

data nameamphatamines;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = MAS-IR
Brand02 = MAS-SR
;

run; quit;


/**************************************************************/

data dextroamphetamine  (drop = lpoii6-lpoii7 lpzii6-lpzii7);
   set aids2 (keep = fips year u6-u7 uo6-uo7  lpoii6-lpoii7  lpzii6-lpzii7  lpomi3 lpomii3 lpzmi3 lpzmii3
                      yy3 t1 t2 lpo1-lpo17 lpoz1-lpoz17);
   rename 
   yy3     = y           
   lpomi3  = lpi
   lpomii3 = lpii
   lpzmi3  = lpzi
   lpzmii3 = lpzii
   uo6    = s01
   uo7    = s02
   u6     = s1
   u7     = s2
   ;

   lp1   = lpoii6;
   lp2   = lpoii7;

   lpz1  = lpzii6;
   lpz2  = lpzii7;

run; quit;   

  data dextroamphetamine;
   set dextroamphetamine;
   if y = 0 or y = . then delete;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit; 

proc sort data = dextroamphetamine; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data dextroamphetamine; merge dextroamphetamine(in=a) demo(in=b);  by fips year; 
if a = 1 then output;
run; quit;
%setfirstyr(dextroamphetamine,2000);                 /*Choose first year of the Panel */
%panelv1(dextroamphetamine,4);                       /*Balance the panel */


data namedextroamphetamine;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = DEX-IR
Brand02 = DEX-ER
;

run; quit;



/**************************************************************/

data alldrugs  (drop = lpomii1-lpomii4 lpzmii1-lpzmii4 q1-q17);
   set aids2 (keep = fips year cenregc lpoi lpoii lpzi lpzii mo1-mo4 m1-m4 lpomii1-lpomii4
                     lpzmii1-lpzmii4 q1-q17 
                      rr t1 t2 yy1 yy2 yy3 yy4 lpo1-lpo17 lpoz1-lpoz17 
                      u1-u8);
   rename 
   rr     = y           
   lpoi   = lpi
   lpoii  = lpii
   lpzi   = lpzi
   lpzii  = lpzii
   mo1    = s01
   mo2    = s02
   mo3    = s03
   mo4    = s04
   m1     = s1
   m2     = s2
   m3     = s3 
   m4     = s4
   ;

   lp1   = lpomii1;
   lp2   = lpomii2;
   lp3   = lpomii3;
   lp4   = lpomii4;

   lpz1  = lpzmii1;
   lpz2  = lpzmii2;
   lpz3  = lpzmii3;
   lpz4  = lpzmii4;

  if yy1~= 0 then lnq1a = log(yy1) - lp1;
  if yy2~= 0 then lnq2a = log(yy2) - lp2;
  if yy3~= 0 then lnq3a = log(yy3) - lp3;
  if yy4~= 0 then lnq4a = log(yy4) - lp4;



   q1b    = sum(q1,q2,q3,q4,q5,q6,q7,q8);
   q2b    = sum(q9,q10,q11);
   q3b    = sum(q12,q13,q14,q15,q16);
   q4b    = q17;

   if q1b ~= 0 then   lnq1b = log(q1b);
   if q2b ~= 0 then   lnq2b = log(q2b);
   if q3b ~= 0 then   lnq3b = log(q3b);
   if q4b ~= 0 then   lnq4b = log(q4b);

  lnqb = log(sum(q1b,q2b,q3b,q4b));

run; quit;   

  data alldrugs;
   set alldrugs;
   if y = 0 or y = . then delete;
   ly = log(y);
   lypi = ly - lpi;
   lypii = ly - lpii;
   lypzi = ly - lpzi;
   lypzii = ly - lpzii;
   t1ly = t1*ly;
   t2ly = t2*ly;

   t1lpzii = t1*lpzii;
   t2lpzii = t2*lpzii;


   array t1lpoz t1lpoz1-t1lpoz17;
   array t2lpoz t2lpoz1-t2lpoz17;
   array lpoz lpoz1-lpoz17;
   do i = 1 to 17;
     t1lpoz{i} = t1*lpoz{i};
     t2lpoz{i} = t2*lpoz{i};
   end;


run; quit; 

proc sort data = alldrugs; by fips year; run; quit;
proc sort data = demo; by fips year; run; quit;
data alldrugs; merge alldrugs(in=a) demo(in=b);  by fips year; 
  pop = poptot;
  lw = log(pcpi*pop);
  if lw = . then delete;
  if a = 1 then output;
run; quit;
%setfirstyr(alldrugs,2000);                 /*Choose first year of the Panel */
%panelv1(alldrugs,4);                       /*Balance the panel */

/*
data alldrugs;
  set alldrugs;
  lpoz17dt = lpoz17d; if lpoz17dt = . then lpoz17dt = 0;
  lpoz11t = lpoz11; if lpoz11t = . then lpoz11t = 0;

  lpoz15t = lpoz15; if lpoz15t = . then lpoz15t = 0;
  lpoz16t = lpoz16; if lpoz16t = . then lpoz16t = 0;


  lpoz17dt = lpoz17dt*t1;
  lpoz11t = lpoz11t*t1;
  lpoz8t = lpoz8*t1;
  lpoz15t = lpoz15t*t1;
  lpoz16t = lpoz16t*t1;


run; quit;
*/


data namealldrugs;
infile datalines dsd dlm='=';
input brand $ description $65.;
datalines ;
Brand01 = MPH
Brand02 = MAS
Brand03 = DEX
Brand04 = OFF
;

run; quit;


proc datasets lib =  work nolist;
 delete prefirstyr:;
run; quit;
