


%let progspath =I:\bokhari\fb_research\analysis\adhd\training\progs\;
%include  "&progspath\mk-AIDS-segments.sas";
%include  "&progspath\macros-elasticities-ver05.sas";



filename myfile1 "I:\bokhari\fb_research\analysis\adhd\training\progs\results01-model-estimate-AIDS-ver01.html";
filename myfile2 "I:\bokhari\fb_research\analysis\adhd\training\progs\results02-model-estimate-AIDS-ver01.html";
%let modnum    = betas;    /*name of the file where to save coeffeicients and standard errors */
%let marshcat  = marsh;     /*name of SAS catalog where unconditional elasticites will be saved*/



/********************** DEFINE MACRO ELSTICITYATV1 ***************************

Define the macro: elasticityatv1(arg#1,arg#2,arg#3,arg#4) where
    arg#1 --> Name of the input dataset
    arg#2 --> Name of the output dataset containing mean shares to be later used
    arg#3 --> Number of drugs in the segment
    arg#4 --> Value of the year for which means need to be computed.


Purpose: Reads in the dataset named in arg#1 and computes the mean share of each
drug in the segment.  The mean share is either the mean over all counties and
all years, or over all counties and a specific year (say 2003).  Theses means
are then sent to the dataset named in arg#2 (and the dataset can later be
called upon to compute elasticities which need to be computed at specific share
or specific average shares.

Note that legitimate values of arg#4 are either a value of the year (such as 2003)
or just a dot, i.e., "." (without quotes).  A dot implies that the average is to
be computed over all counties and all years.


Example1: elasticityatv1(segment101,meanshares,3,2003). Reads in the dataset
work.segment101.  Computes the mean of s1-s3 for the year 2003 (from all counties)
and sends output to a dataset meanshares where the names of the means are s01-s03.

Example2: elasticityatv1(segment101,meanshares,3,.). Reads in the dataset
work.segment101.  Computes the mean of s1-s3 for all years (from all counties)
and sends output to a dataset meanshares where the names of the means are s01-s03.



******************************************************************************/



%MACRO ELASTICITYATV1(INDATA,OUTDATA,VARS,YEAR);

proc means data = &indata noprint ;
 var s1-s&vars s01-s0&vars;
 class year;
 output out = &outdata mean = s1-s&vars s01-s0&vars;
 run; quit;


data &outdata (drop = _type_ _freq_ year);
  set &outdata;
  if year = &year then output;
run; quit;

%MEND;


%let myprint   = noprint;

/****************** STEP 1: ESTIMATE LEVEL 1 (BOTTOM) SEGMENT M1F1 **************
   M1F1:
   (1) Segment with three drugs.
   (2) Input data is in SAS DATA file work.segment101.
   (3) Estimated coefficients and covariances are saved in SAS datasets
       work.m1f1results and work.m1f1resultsB.
   (4) The 'elasticityatv1' macro creates a small dataset 'm1f1shares' with
       mean shares of the three drugs so that elasticity can be computed using
       the values in 'm1f1shares'.
   (5) Post estimation macro 'elasticity3b' computes and saves elasticities
       as matrices in SAS CATALOGS m1f1ols and m1f1iv (for later printing).
*******************************************************************************/

%elasticityatv1(segment101,m1f1shares,3,.);
    proc model data=segment101 noprint;

    /*Omit Last Share Equation  for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1   +  c12*t2 + c13*lnkids + c14*lnmds  +  c15*lncaiddrugs + c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 + g13*lp3 ;
    s2 = a2 + b2*lypi +  c21*t1 +  c22*t2 +  c23*lnkids + c24*lnmds + c25*lncaiddrugs +  c26*lnmcaidenrollees +
         g21*lp1 + g22*lp2 + g23*lp3 ;
    fit s1 s2 / sur 3sls hausman converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12 + g13  = 0;
    restrict g21 + g22 + g23  = 0;
   /*Symmetry Restriction*/
    restrict g12 = g21;

	/* Save estimated coefficients and covariances on prices and expenditures in work.m1f1results*/
	estimate b1, g11, g12, g13,
             b2, g21, g22, g23,
            'b3' -(b1+b2),
            'g31' g13,
            'g32' g23,
            'g33' -(g13 + g23),
    /outest=m1f1results outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.m1f1resultsB*/
    estimate c11, c12, c13, c14, c15, c16,
			 c21, c22, c23, c24, c25, c26,
			'c31'  - (c11 + c21),
            'c32'  - (c12 + c22),
            'c33'  - (c13 + c23),
            'c34'  - (c14 + c24),
            'c35'  - ( c15 + c25),
			'c36'  - (c16 + c26),
    /outest=m1f1resultsb outcov;

    endogenous s1 s2 lypi lp1 lp2 lp3;
    instruments  lpz1 lpz2 lpz3 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly;
run; quit;
%elasticity3b(namesegment101,m1f1results,m1f1shares,'SUR',m1f1ols,print);    /*Compute elasticities from OLS */
%elasticity3b(namesegment101,m1f1results,m1f1shares,'3SLS',m1f1iv,print);    /*Compute elasticities from 2SLS */
proc datasets lib=work nolist; delete m1f1shares; run; quit;


/****************** STEP2: ESTIMATE LEVEL 1 (BOTTOM) SEGMENT M1F2 ********************
   M1F2:
   (1) Segment with four drugs.
   (2) Input data is in SAS DATA file work.segment122.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.m1f2results and work.m1f2resultsB.
   (3) The 'elasticityatv1' macro creates a small dataset 'm1f2shares' with
       mean shares of the four drugs so that elasticity can be computed using
       values in 'm1f2shares'.
   (4) Post estimation macro 'elasticity4b' computes and saves elasticities
       as matrices in SAS CATALOGS m1f2ols and m1f2iv (for later printing).
*******************************************************************************/

%elasticityatv1(segment122,m1f2shares,4,.);
proc model data=segment122 noprint ;

    /*Omit Last Share Equation  for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1 + c12*t2 + c13*lnkids + c14*lnmds + c15*lncaiddrugs + c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 + g13*lp3 + g14*lp4 ;

    s2 = a2 + b2*lypi + c21*t1  + c22*t2 + c23*lnkids + c24*lnmds + c25*lncaiddrugs + c26*lnmcaidenrollees +
         g21*lp1 + g22*lp2 + g23*lp3 + g24*lp4 ;

    s3 = a3 + b3*lypi + c31*t1 + c32*t2 + c33*lnkids + c34*lnmds + c35*lncaiddrugs + c36*lnmcaidenrollees  +
         g31*lp1 + g32*lp2 + g33*lp3 + g34*lp4 ;

    fit s1 s2 s3 / sur 3sls hausman  converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12 + g13 + g14  = 0;
    restrict g21 + g22 + g23 + g24  = 0;
    restrict g31 + g32 + g33 + g34  = 0;
   /*Symmetry Restriction*/
    restrict g12 = g21;
    restrict g13 = g31;
    restrict g23 = g32;

	/* Save estimated coefficients and covariances on prices and expenditures in work.m1f2results*/
    estimate b1, g11, g12, g13, g14,
             b2, g21, g22, g23, g24,
             b3, g31, g32, g33, g34,
            'b4' -(b1+b2+b3),
            'g41' g14,
            'g42' g24,
			'g43' g34,
            'g44' -(g14 + g24 + g34),
     /outest=m1f2results outcov;

     /* Save estimated coefficients and covariances on exogenous variables in work.m1f2resultsB*/
     estimate c11, c12, c13, c14, c15, c16,
			  c21, c22, c23, c24, c25, c26,
              c31, c32, c33, c34, c35, c36,
			 'c41' - (c11 + c21 + c31),
			 'c42' - (c12 + c22 + c32),
             'c43' - (c13 + c23 + c33),
			 'c44' - (c14 + c24 + c34),
			 'c45' - (c15 + c25 + c35),
			 'c46' - (c16 + c26 + c36),
     /outest=m1f2resultsb outcov ;


   endogenous s1-s3 lp1-lp4 lypi ;
   instruments  lpz1 lpz2 lpz3 lpz4 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly;
   run; quit ;

%elasticity4b(namesegment122,m1f2results,m1f2shares,'SUR',m1f2ols,print);    /*Compute elasticities from OLS */
%elasticity4b(namesegment122,m1f2results,m1f2shares,'3SLS',m1f2iv,print);    /*Compute elasticities from 2SLS */
proc datasets lib=work nolist; delete m1f2shares; run; quit;



/******************STEP 3: ESTIMATE LEVEL 2 (MIDDLE) SEGMENT M1 ***********************
   M1:
   (1) Segment with three (aggregate) drugs.
   (2) Input dta is in SAS DATA file work.methylphenidates.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.m1results and work.m1resultsB.
   (4) The 'elasticityatv1' macro creates a small dataset 'm1shares' with
       mean shares of the three drugs so that elasticity can be computed using
       values in 'm1shares'.
   (5) Post estimation macro 'elasticity3b' computes and saves elasticities
       as matrices in SAS CATALOGS m1ols and m1iv (for later printing).
*******************************************************************************/

%elasticityatv1(methylphenidates,m1shares,3,.);
proc model data=methylphenidates noprint;

    /*Omit Last Share Equation  for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1 +  c12*t2  + c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 + g13*lp3 ;

    s2 = a2 + b2*lypi +  c21*t1 +  c22*t2 + c23*lnkids + c24*lnmds + c25*lncaiddrugs + 	c26*lnmcaidenrollees +
         g21*lp1 + g22*lp2 + g23*lp3 ;

    fit s1 s2 / sur 3sls hausman  converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12 + g13  = 0;
    restrict g21 + g22 + g23  = 0;
   /*Symmetry Restriction*/
    restrict g12 = g21;

	/* Save estimated coefficients and covariances on prices and expenditures in work.m1results*/
	estimate b1, g11, g12, g13,
             b2, g21, g22, g23,
            'b3' -(b1+b2),
            'g31' g13,
            'g32' g23,
            'g33' -(g13 + g23),
			/outest=m1results outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.m1resultsB*/
	estimate
 			c11, c12, c13, c14, c15, c16,
			c21, c22, c23, c24, c25, c26,
            'c31' - (c11 + c21),
            'c32' - (c12 + c22),
            'c33' - (c13 + c23),
            'c34' - (c14 + c24),
            'c35' - (c15 + c25),
			'c36' - (c16 + c26),
            /outest=m1resultsb outcov;


    endogenous s1 s2 lypi lp1 lp2 lp3 ;
    instruments  lpz1 lpz2 lpz3 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly
                 lpoz1-lpoz3 lpoz6-lpoz7 lpoz4 lpoz5
    ;

  run;
  quit ;
%elasticity3b(namemethylphenidates,m1results,m1shares,'SUR',m1ols,print);     /*Compute elasticities from SUR */
%elasticity3b(namemethylphenidates,m1results,m1shares,'3SLS',m1iv,print);    /*Compute elasticities from 3SLS */
proc datasets lib=work nolist; delete m1shares; run; quit;


/******************STEP4:  ESTIMATE LEVEL 1 (BOTTOM) SEGMENT M2F1 ***********************
   M2F1:
   (1) Segment with two drugs.
   (2) Input data is in data file work.segment201.
   (3) Estimated coefficients and covariances are saved in SAS DATASATES
       work.m2f1results and work.m2f1resultsB.
   (3) The 'elasticityatv1' macro creates a small dataset 'm2f1shares' with
       mean shares of the two drugs so that elasticity can be computed using
       values in 'm2f1shares'.
   (4) Post estimation macro 'elasticity2b' computes and saves elasticities
       as matices in SAS CATALOGS m2f1ols and m2f1iv (for later printing).
*******************************************************************************/


%elasticityatv1(segment201,m2f1shares,2,.);
proc model data=segment201 noprint;
    /*Omit Last Share Equation for adding up restrictions */

    s1 = a1 + b1*lypi +  c11*t1 /* + c12*t2 */  + c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 ;

    fit s1 / sur 3sls hausman converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12  = 0;
   /*Symmetry Restriction*/

	/* Save estimated coefficients and covariances on prices and expenditures in work.m2f1results*/
    estimate b1, g11, g12,
            'b2' -(b1),
            'g21' g12,
            'g22' -(g12),
    /outest=m2f1results outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.m2f1resultsB*/
	estimate c11, c13, c14,	c15, c16,
			 'c21' - (c11),
             'c23' - (c13),
             'c24' - (c14),
             'c25'  - (c15),
			 'c26' - (c16),
    /outest=m2f1resultsb outcov;


   endogenous s1 lp1 lp2 lypi;
   instruments  lpz1 lpz2 lpzi
                 t1 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly
                ;
 run; quit ;


%elasticity2b(namesegment201,m2f1results,m2f1shares,'SUR',m2f1ols,print);     /*Compute elasticities from SUR */
%elasticity2b(namesegment201,m2f1results,m2f1shares,'3SLS',m2f1iv,print);     /*Compute elasticities from 3SLS */

/*********************************************************************************/



/******************STEP5:  ESTIMATE LEVEL 2 (MIDDLE) SEGMENT M2 ***********************
   M2:
   (1) Segment with two (aggregate) drugs.
   (2) Input data is in SAS DATA file work.amphatamines.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.m2results and work.m2resultsB.
   (4) The 'elasticityatv1' macro creates a small dataset 'm2shares' with
       mean shares of the two drugs so that elasticity can be computed using
       the values in 'm2shares'.
   (5) Post estimation macro 'elasticity2b' computes and saves elasticities
       as matrices in SAS CATALOGS m2ols and m2iv (for later printing).
*******************************************************************************/

%elasticityatv1(amphatamines,m2shares,2,.);
proc model data=amphatamines noprint;

   /*Omit Last Share Equation for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1 + c12*t2 +  c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 ;

    fit s1 / sur 3sls hausman converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12  = 0;
   /*Symmetry Restriction*/

	/* Save estimated coefficients and covariances on prices and expenditures in work.m2results*/
    estimate b1, g11, g12,
            'b2' -(b1),
            'g21' g12,
            'g22' -(g12),
    /outest=m2results outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.m2resultsB*/
    estimate  c11, c12, c13, c14, c15, c16,
             'c21' - (c11),
			 'c22' - (c12),
             'c23' - (c13),
             'c24' - (c14),
             'c25' - (c15),
			 'c26' - (c16),
   /outest=m2resultsb outcov;


   endogenous s1 lp1 lp2 lypi;
   instruments  lpz1 lpz2 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly
				 lpoz9;

run;
quit ;

%elasticity2b(nameamphatamines,m2results,m2shares,'SUR',m2ols,print);     /*Compute elasticities from OLS */
%elasticity2b(nameamphatamines,m2results,m2shares,'3SLS',m2iv,print);    /*Compute elasticities from 3SLS */
proc datasets lib=work nolist; delete m2shares; run; quit;



/******************STEP6:  ESTIMATE LEVEL 1 (BOTTOM) SEGMENT M3F1 *********************
   M3F1:
   (1) Segment with three drugs.
   (2) Inout data is in SAS DATA file work.segment300.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.m3f1results and work.m3f1resultsB.
   (3) The 'elasticityatv1' macro creates a small dataset 'm3f1shares' with
       mean shares of the three drugs so that elasticity can be computed using
       values in 'm3f1shares'.
   (4) Post estimation macro 'elasticity3b' computes and saves elasticities
       as matrices in SAS CATALOGS m3f1ols and m3f1iv (for later printing).
*******************************************************************************/

%elasticityatv1(segment300,m3f1shares,3,.);
 proc model data=segment300 noprint;
    /*Omit Last Share Equation  for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1 +  c12*t2 + c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 + g13*lp3 ;

    s2 = a2 + b2*lypi +  c21*t1 +   c22*t2 + c23*lnkids + c24*lnmds +  c25*lncaiddrugs + 	c26*lnmcaidenrollees +
         g21*lp1 + g22*lp2 + g23*lp3 ;

    fit s1 s2 / sur 3sls hausman  converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12 + g13  = 0;
    restrict g21 + g22 + g23  = 0;


   /*Symmetry Restriction*/
    restrict g12 = g21;

   /* Save estimated coefficients and covariances on prices and expenditures in work.m2f1results*/
	estimate b1, g11, g12, g13,
             b2, g21, g22, g23,
            'b3' -(b1+b2),
            'g31' g13,
            'g32' g23,
            'g33' -(g13 + g23),
    /outest=m3f1results outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.m3f1resultsB*/
	estimate c11, c12, c13, c14, c15, c16,
			 c21, c22, c23, c24, c25, c26,
            'c31' - (c11 + c21),
            'c32' - (c12 + c22),
            'c33' - (c13 + c23),
            'c34' - (c14 + c24),
            'c35' - (c15 + c25),
			'c36' - (c16 + c26),
    /outest=m3f1resultsb outcov;


    endogenous s1 s2 lypi lp1 lp2 lp3;
    instruments  lpz1 lpz2 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly;

run; quit ;
%elasticity3b(namesegment300,m3f1results,m3f1shares,'SUR',m3f1ols,print);     /*Compute elasticities from OLS */
%elasticity3b(namesegment300,m3f1results,m3f1shares,'3SLS',m3f1iv,print);    /*Compute elasticities from 2SLS */



/******************STEP7: ESTIMATE LEVEL 1 (BOTTOM) SEGMENT M3F2 *********************
   M3F2:
   (1) Segment with two drugs.
   (2) Input data is in SAS DATA file work.segment301.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.m3f2results and work.m2f3resultsB.
   (3) The 'elasticityatv1' macro creates a small dataset 'm3f2shares' with
       mean shares of the three drugs so that elasticity can be computed using
       values in 'm3f2shares'.
   (4) Post estimation macro 'elasticity2b' computes and saves elasticities
       as matrices in SAS CATALOGS m3f2ols and m3f2iv (for later printing).
*******************************************************************************/

%elasticityatv1(segment301,m3f2shares,2,.);
proc model data=segment301 noprint;

    /*Omit Last Share Equation for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1  /* + c12*t2 */ + c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 ;

    fit s1 / sur 3sls hausman converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12  = 0;
   /*Symmetry Restriction*/

	/* Save estimated coefficients and covariances on prices and expenditures in work.m3f2results*/
    estimate b1, g11, g12,
            'b2' -(b1),
            'g21' g12,
            'g22' -(g12),
   /outest=m3f2results outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.m3f2resultsB*/
     estimate c11, c13, c14, c15, c16,
            'c21' - (c11),
            'c23' - (c13),
            'c24' - (c14),
            'c25'  - (c15),
			'c26'  - (c16),
     /outest=m3f2resultsb outcov;


   endogenous s1 lp1 lp2 lypi;
   instruments  lpz1 lpz2 lpzi
                 t1 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly                 ;
run; quit ;
%elasticity2b(namesegment301,m3f2results,m3f2shares,'SUR',m3f2ols,print);     /*Compute elasticities from SUR */
%elasticity2b(namesegment301,m3f2results,m3f2shares,'3SLS',m3f2iv,print);    /*Compute elasticities from 3SLS */


/******************STEP8:  ESTIMATE LEVEL 2 (MIDDLE) SEGMENT M3 *********************
   M3:
   (1) Segment with two (aggregate) drugs.
   (2) Input data is in SAS DATA file work.dextroamphetamine.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.m3results and work.m3resultsB.
   (4) The 'elasticityatv1' macro creates a small dataset 'm3shares' with
       mean shares of the three drugs so that elasticity can be computed using
       values in 'm3shares'.
   (5) Post estimation macro 'elasticity2b' computes and saves elasticities
       in SAS CATALOGS m3ols and m3iv (for later printing).
*******************************************************************************/

%elasticityatv1(dextroamphetamine,m3shares,2,.);
proc model data=dextroamphetamine noprint ;

    /*Omit Last Share Equation for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1   +   c12*t2 + c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 ;

    fit s1 / SUR 3sls hausman converge = .00001;

    /* Homogenity restriction  */
    restrict g11 + g12  = 0;
   /*Symmetry Restriction*/

	/* Save estimated coefficients and covariances on prices and expenditures in work.m3results*/
    estimate b1, g11, g12,
            'b2' -(b1),
            'g21' g12,
            'g22' -(g12),
   /outest=m3results outcov;

	 /* Save estimated coefficients and covariances on exogenous variables in work.m3resultsB*/
     estimate c11, c12, c13, c14, c15, c16,
			 'c21' - (c11),
			 'c22' - (c12),
             'c23' - (c13),
             'c24' - (c14),
             'c25' - (c15),
			 'c26' - (c16),
     /outest=m3resultsb outcov;

   endogenous s1 lp1 lp2 lypi;
   instruments  lpz1 lpz2 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly 
                 lpoz12 lpo13 lpoz14 lpoz15;

run; quit ;
%elasticity2b(namedextroamphetamine,m3results,m3shares,'SUR',m3ols,print);     /*Compute elasticities from SUR */
%elasticity2b(namedextroamphetamine,m3results,m3shares,'3SLS',m3iv,print);    /*Compute elasticities from 3SLS */


/******************STEP9:  ESTIMATE LEVEL 3 (UPPER-MIDDLE) SEGMENT M*******************
   M:
   (1) Segment with four (aggregate) drugs (i.e., molecules).
   (2) Input data is in SAS DATA file work.alldrugs.
   (3) Estimated coefficients and covariances are saved in SAS DATASETS
       work.mresults and work.mresultsB.
   (3) The 'elasticityatv1' macro creates a small dataset 'mshares' with
       mean shares of the four drugs so that elasticity can be computed using
       values in 'mshares'.
   (4) Post estimation macro 'elasticity4b' computes and saves elasticities
       as matrices in SAS catalogs mols and miv (for later printing).
*******************************************************************************/


%elasticityatv1(alldrugs,mshares,4,.);
proc model data = alldrugs ; 
   lnq1b = a1 + b1*ly + g11*lp1 + g12*lp2 + g13*lp3 + g14*lp4 +  c11*t1 + c12*t2  + c13*lnkids + c14*lnmds + c15*lncaiddrugs +  c16*lnmcaidenrollees +  c18*u3  + c19*u5   ;
   lnq2b = a2 + b2*ly + g21*lp1 + g22*lp2 + g23*lp3 + g24*lp4 +  c21*t1 + c22*t2  + c23*lnkids + c24*lnmds + c25*lncaiddrugs +  c26*lnmcaidenrollees +  c28*u3  + c29*u5   ;
   lnq3b = a3 + b3*ly + g31*lp1 + g32*lp2 + g33*lp3 + g34*lp4 +  c31*t1 + c32*t2  + c33*lnkids + c34*lnmds + c35*lncaiddrugs +  c36*lnmcaidenrollees +  c38*u3  + c39*u5   ;
   lnq4b = a4 + b4*ly + g41*lp1 + g42*lp2 + g43*lp3 + g44*lp4 +  c41*t1 + c42*t2  + c43*lnkids + c44*lnmds + c45*lncaiddrugs +  c46*lnmcaidenrollees +  c48*u3  + c49*u5   ;

   fit lnq1b lnq2b lnq3b lnq4b / ols 2sls hausman  converge = .00001;


  /*Symmetry Restriction  -- data on fourth aggregate drug is not well behaved. Do not include in restrictions*/
    restrict g12 = g21;
    restrict g13 = g31;
  * restrict g14 = g41;
    restrict g23 = g32;
  *	restrict g24 = g42;
  * restrict g34 = g43;

  /* Homogenity in income and prices*/
    restrict b1 + g11 + g12 + g13 + g14 = 0;
	restrict b2 + g21 + g22 + g23 + g24 = 0;
	restrict b3 + g31 + g32 + g33 + g34 = 0;
	restrict b4 + g41 + g42 + g43 + g44 = 0;

	/* Save estimated coefficients and covariances on prices and expenditures in work.mresults*/
    estimate b1, g11, g12, g13, g14,
             b2, g21, g22, g23, g24,
             b3, g31, g32, g33, g34,
             b4, g41, g42, g43, g44,
	/outest=mresults outcov;

	/* Save estimated coefficients and covariances on exogenous variables in work.mresultsB*/
	estimate c11, c12, c13, c14, c15, c16, c18, c19,
			 c21, c22, c23, c24, c25, c26, c28, c29,
             c31, c32, c33, c34, c35, c36, c38, c39,
             c41, c42, c43, c44, c45, c46, c48, c49,
    /outest=mresultsb outcov;



   endogenous lnq1b lnq2b lnq3b lnq4b lp1 lp2 lp3 lp4 ;
   instruments lpz1 lpz2 lpz3 lpz4 u3 u5
               lpoz1-lpoz3 lpoz6-lpoz9 lpoz12 lpoz13 lpoz15
               t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
               ly t1ly t2ly  
               t1lnkids t1lnmds t1lncaiddrugs t1lnmcaidenrollees
			   t2lnkids t2lnmds t2lncaiddrugs t2lnmcaidenrollees;
 run; quit ;
%elasticity4Blnq(namealldrugs,mresults,mshares,'OLS',mols,print);     /*Compute elasticities from OLS (not SUR) */
%elasticity4Blnq(namealldrugs,mresults,mshares,'2SLS',miv,print);     /*Compute elasticities from 2SLS (not 3SLS) */


/******************STEP10:  ESTIMATE LEVEL 4 (TOP) SEGMENT (All Srugs)******************
   TOP:
   (1) Segment with 1 (aggregate) drug (i.e., total quantity of all drugs).
   (2) Input data is in SAS DATA file work.alldrugs.
   (3) Save all estimated coefficients and covariances in SAS DATA file
       'work.topresults'.
*******************************************************************************/



/* Also estimate the top level model */

proc model data = alldrugs print;
  lnqb = a1 + b1*lw + g11*lpii +  c11*t1 +  c12*t2 + c13*lnkids + c14*lnmds + c15*lncaiddrugs  +  c16*lnmcaidenrollees  ;

  /* Save all coefficients and covariances in work.topresults*/
  fit lnqb/ols 2sls hausman  converge = .00001 outcov outest= topresults converge = .00001;

  endogenous lnqb lpii;
  exogenous lpzii
            t1 t2  lnkids lnmds lncaiddrugs lnmcaidenrollees lw
            lpoz1-lpoz3 lpoz6-lpoz9 lpoz12 lpoz13 lpoz15;
run; quit;


/********************************END OF STEPS1-thru-STEP10*************************************
 The estimations are all done.  Coefficients are saved in various (temp) datasets
 and elasticities are saved in various (temp) catalogs. 
 The rest of the promram does two main things:

     1.  Combine various earlier results and print them in a nicer way (conditional elasticties
         regression coefficients etc.)
     2. Based on the regression coefficients from various segments, compute the full unconditional
        elasticity matrix between all drugs.  The formula for computing the unconditional
        elasticities is derived and given in Bokhari and Fournier 2013.

*****************************************************************************************/



/************* STEP11: CLEANUP RESULTS OF TOP EQUATION **********************************

11.a Read in results from top equation (saved in 'topresults') and seperate OLS and 2SLS
     into seperate data files (topols and topiv).
11.b Read in datasets 'topols' and 'topiv' and save the coefficients as matrices in
     catalogs 'topols' and 'topiv'.

***************************************************************************************/

/*Step 11.a*/
data topols (drop = _name_ _type_ _status_ _nused_);
  set topresults ;
  if _type_ = 'OLS' and _name_ = '' then output;
  run; quit;


data topiv (drop = _name_ _type_ _status_ _nused_);
  set topresults ;
  if _type_ = '2SLS' and _name_ = '' then output;
  run; quit;

/*Step 11.b*/
proc iml;
  use topols;
   read all into coeffs[colname = varname];
   read all var{a1} into  a;
   read all var{b1} into  b;
   read all var{g11}into  g;
   read all var{c11 c12 c13 c14 c15 c16} into c;
  close topols;
   reset storage = work.topols;
   remove _all_;
   store a b g c;
quit;

proc iml;
 use topiv;
   read all into coeffs[colname = varname];
   read all var{a1} into  a;
   read all var{b1} into  b;
   read all var{g11}into  g;
   read all var{c11 c12 c13 c14 c15 c16}  into c;
  close topiv;
   reset storage = work.topiv;
   remove _all_;
   store a b g c;
 quit;

/********************************* END: STEP 11********************************************/
/******************************************************************************************/



/************************************* STEP 12: *******************************************

STEP 12:   Combine all the previous results (coefficents and elasticities) into fewer
           catalogs and then compute the unconditional elasticities. Also print all the
           conditional and unconditional elasticities. 
     
STEPS
   12.a:  Combine all IV based coefficents and average shares into one catalog ONEIV.
          Need all these for computing IV based uncondtional elasticties matrix. 
   12.b:  Combine all OLS based coefficents and average shares into one catalog ONEOLS.
          Need all these for computing OLS based uncondtional elasticties matrix. 
   12.c:  Combine all IV based conditional elasticties (Marshallian, Hicksian and
          Hickes-Allen which were previously computed and saved in different catalogs)
          into a single catalog named TWOIV.
   12.d:  Combine all OLS based conditional elasticties (Marshallian, Hicksian and
          Hickes-Allen which were previously computed and saved in different catalogs)
          into a single catalog named TWOOLS.
   12.e:  Based on regression coefficients and shares in CATALOGS ONEIV (or ONEOLS)
          this step computes an UNCONDITIONAL elasticity matrix. Additionally it 
          also reads in CATALOG TWOIV (or TWOOLS) to print all the conditional 
          elasticities. 



 STEP 12.a:  Combine all IV based coefficents on prices, expenditures and mean 
             shares into a single SAS CATALOG named 'ONEIV'.  
  
      The vaious CATALOGS saved earlier (m1f1iv, m1f2iv etc.) all have matrices saved
      in them with names b, g, s and s0. These names correspond to:

      b --> coefficients on expenditure in each equation (from a given segment)
      g --> coefficients on log prices in each equation (from a given segment)
      s --> share values at which elasticities were computed
      s0 --> share values at which Stone price indexes were created. 

      Thus, read in each orginal IV (or OLS) based CATALOG, append the name of the 
      segment/catalog to the matrices b,g,s,s0 and save all of them into a single
      catalog named 'oneiv.  Since this is a repetivie task, do so usint a macro.

STEP 12.b:  Same as STEP 12.a except do it for OLS/SUR based estimates. Create 
     a combined catalog named 'ONEOLS'.

STEP 12.c: Combine all IV based conditional elasticities (marshallian, hicksian, 
           hicksallen, expenditures), mean shares, name of products (in vector 'key')
           into a single SAS CATALOG named 'TWOIV'.  

      The vaious CATALOGS saved earlier (m1f1iv, m1f2iv etc.) all have matrices saved
      in them with names like ehij, emij, haij, etap, key, s.  These names correspond to:

      emij --> Marshallian Elasticity
      ehij --> Compensated (Hicksian) Elasticity
      haij --> Hicks-Allen measures of elasticity
      etap --> Expenditure elasticities 
      s    --> Share values at which elasticities were computed (avg values)
      key  --> Brand descriptions 

      Thus, read in each orginal IV (or OLS) based CATALOG, append the name of the 
      segment/catalog to the matrices emij,ehij etc. matrices above and save all of
      them into a single catalog named 'TWOIV'.  
      Since this is a repetivie task, do so using a macro.

STEP 12.d. Same as 12.c above, except to it for OLS/SUR based estimates. Create 
     a combined catalog named 'TWOOLS'. 


 STEP 12.e.  This step is written as an IML macro. It reads in all the regression 
      coefficients (and relative shares) from all the different segments (M1F1, M1F2
      ect.) to compute the unconditional matrix of elasticities. Advantage of IML 
      macro is that it can be sequentially fed the OLS coefficients and OLS conditional
      elasticities to print all OLS based results and then repeat the process with IV
      based estimates.  

***************************************************************************************/




/***************************************************************************************
 DEFINE A MACRO named 'mk' to help do steps 12a and 12b repeately
 **************************************************************************************/
 %macro mk(seg,cat,catout);
proc iml;
start readbgs;
 	reset storage = work.&cat;
 	load b g s s0;
 	b_&seg = b;
 	g_&seg = g;
 	s_&seg = s;
 	s0_&seg = s0;

reset storage = work.&catout;
store b_&seg g_&seg s_&seg s0_&seg;
finish;

run readbgs;
quit;

%mend;
/***************************************************************************************
 END OF MACRO DEFINATION 
***************************************************************************************/


/************************************ STEP 12.a: ***************************************
Combine all previous IV parameter estimates into a a single catalog work.oneiv
****************************************************************************************/
 proc iml; reset storage = work.oneiv; remove _all_;  quit;
%mk(m1f1,m1f1iv,oneiv);
%mk(m1f2,m1f2iv,oneiv);
%mk(m2f1,m2f1iv,oneiv);
%mk(m3f1,m3f1iv,oneiv);
%mk(m3f2,m3f2iv,oneiv);
%mk(m1,m1iv,oneiv);
%mk(m2,m2iv,oneiv);
%mk(m3,m3iv,oneiv);
%mk(m,miv,oneiv);
/* Also add in the top level IV estimates to this catalog */ 
proc iml;
  reset storage = work.topiv;
  load b g;
  b_top = b;
  g_top = g;
  reset storage = work.oneiv;
  store b_top g_top;
quit;

/************************************ STEP 12.b: ***************************************
Combine all previous OLS parameter estimates into a a single catalog work.oneols
****************************************************************************************/

 proc iml; reset storage = work.oneols; remove _all_;  quit;
%mk(m1f1,m1f1ols,oneols);
%mk(m1f2,m1f2ols,oneols);
%mk(m2f1,m2f1ols,oneols);
%mk(m3f1,m3f1ols,oneols);
%mk(m3f2,m3f2ols,oneols);
%mk(m1,m1ols,oneols);
%mk(m2,m2ols,oneols);
%mk(m3,m3ols,oneols);
%mk(m,mols,oneols);
/* Also add in the top level OLS estimates to this catalog */ 
proc iml;
  reset storage = work.topols;
  load b g;
  b_top = b;
  g_top = g;
  reset storage = work.oneols;
  store b_top g_top;
quit;

/***************************************************************************************
 DEFINE A MACRO named 'mk2' to help do steps 12c and 12d repeately
 **************************************************************************************/

 %macro mk2(seg,cat,catout);
 proc iml;

start elasticity;
 	reset storage = work.&cat;
 	load ehij emij key s etap haij;
 	ehij_&seg = ehij;
	haij_&seg = haij;
 	emij_&seg = emij;
 	key_&seg =  key;
    s_&seg = s;
	etap_&seg = etap;


	reset storage = work.&catout;
	store ehij_&seg emij_&seg haij_&seg key_&seg s_&seg etap_&seg;
finish;

 run elasticity;

 quit;
 %mend;

/************************************ STEP 12.c: ***************************************
Combine all previous conditional IV elasticties into a a single catalog two.iv
so it is easier rearrange them and print them.
****************************************************************************************/

 proc iml; reset storage = work.twoiv; remove _all_;  quit;
%mk2(m1f1,m1f1iv,twoiv);
%mk2(m1f2,m1f2iv,twoiv);
%mk2(m2f1,m2f1iv,twoiv);
%mk2(m3f1,m3f1iv,twoiv);
%mk2(m3f2,m3f2iv,twoiv);
%mk2(m1,m1iv,twoiv);
%mk2(m2,m2iv,twoiv);
%mk2(m3,m3iv,twoiv);
%mk2(m,miv,twoiv);



/************************************ STEP 12.d: ***************************************
Combine all previous conditional OLS elasticties into a a single catalog TWOOLS
so it is easier rearrange them and print them.
****************************************************************************************/

 proc iml; reset storage = work.twools; remove _all_;  quit;
%mk2(m1f1,m1f1ols,twools);
%mk2(m1f2,m1f2ols,twools);
%mk2(m2f1,m2f1ols,twools);
%mk2(m3f1,m3f1ols,twools);
%mk2(m3f2,m3f2ols,twools);
%mk2(m1,m1ols,twools);
%mk2(m2,m2ols,twools);
%mk2(m3,m3ols,twools);
%mk2(m,mols,twools);




/************************************ STEP 12.e: ***************************************
 Compute UNCONDITIONAL elasticties:
(i)  Read in CATALOGS ONEIV and ONEOLS (which consist of regression coefficients and shares)
 to compute the uncondtional elasticities matrices.
(ii) Read in the CATALOGS TWOIV and TWOOLS (which contain the conditional elasticities)
 so they can be printed. 
****************************************************************************************/


ods html file = myfile1 newfile = none;
title1 'Conditional and Unconditioanl Elasticties (3SLS/SUR) at Mean Shares';
title2 'Created via SAS program <model-estimate-AIDS-ver01.sas>';


proc iml;
print " ";
ods html text="<p align='left'><b> Notes on Estimation <b> <br> <br>
Note 1: Includes segment122 (Metadate CD/ER and Ritalin SR/LA cobmined via med-equivalence)<br>
Note 2: Includes segments 503 (Drugs combined via med-equivalence) <br>
Note 3: Fixed Effects: State Level for all segments <br>
</b></p>";

reset storage = &marshcat; remove _all_; close;




start unconditionals;

    /* Coeffienent on prices from various segment:
    g_m1f1: matrix of paramaters on log prices from AIDS estimates from M1F1 (bottom) level 1
    Similar interpretation of g_m1f2 etc. 
    g_m1: matrix of parameters on log prices from level 2 AIDS estimates.
    Similar interpretation on g_m2 etc.
    g_m is the matrix of parameters on log prices from level 3 Cobb-Douglas estimates    
    */
     
	gamma = block(g_m1f1,g_m1f2,0,g_m2f1,0,g_m3f1,g_m3f2,0);
	g = block(g_m1,g_m2,g_m3,0);
	gammacap = g_m;

	s_m1f3 = 1;
	s_m2f2 = 1;
	s_m4f1 = 1;
	s_m4 = 1;

	s0_m1f3 = 1;
	s0_m2f2 = 1;
	s0_m4f1 = 1;
	s0_m4 = 1;

	b_m1f3 = 0;
	b_m2f2 = 0;
	b_m4f1 = 0;
	b_m4 = 0;   

	/*shares within molecule-forms  -- ie at bottom level 1*/
	s = s_m1f1||s_m1f2||s_m1f3||s_m2f1||s_m2f2||s_m3f1||s_m3f2||s_m4f1;
	u = s_m1||s_m2||s_m3||s_m4;   /*shares of forms within molecules */
	v = s_m;                      /* share of molecules */

	s0 = s0_m1f1||s0_m1f2||s0_m1f3||s0_m2f1||s0_m2f2||s0_m3f1||s0_m3f2||s0_m4f1;
	u0 = s0_m1||s0_m2||s0_m3||s0_m4;
	v0 = s0_m;

	betaifm = b_m1f1//b_m1f2//b_m1f3//b_m2f1//b_m2f2//b_m3f1//b_m3f2//b_m4f1;  /*column vector */

	s0b = s0@betaifm ;
	u0fpmp = shape(u0[1],17,3)||shape(u0[2],17,4)||shape(u0[3],17,1)||shape(u0[4],17,2)||
	         shape(u0[5],17,1)||shape(u0[6],17,3)||shape(u0[7],17,2)||shape(u0[8],17,1);

	ufm = shape(u[1],3,17)//shape(u[2],4,17)//shape(u[3],1,17)//shape(u[4],2,17)//shape(u[5],1,17)
          //shape(u[6],3,17)//shape(u[7],2,17)//shape(u[8],1,17);

	c = g_top;


	z = gammacap;

	zcapmmp = (shape(z[1,1],8,8)||shape(z[1,2],8,3)||shape(z[1,3],8,5)||shape(z[1,4],8,1))//
    	      (shape(z[2,1],3,8)||shape(z[2,2],3,3)||shape(z[2,3],3,5)||shape(z[2,4],3,1))//
        	  (shape(z[3,1],5,8)||shape(z[3,2],5,3)||shape(z[3,3],5,5)||shape(z[3,4],5,1))//
              (shape(z[4,1],1,8)||shape(z[4,2],1,3)||shape(z[4,3],1,5)||shape(z[4,4],1,1));


	gammacapmmp = (shape(g_m[1,1],8,8)||shape(g_m[1,2],8,3)||shape(g_m[1,3],8,5)||shape(g_m[1,4],8,1))//
    	          (shape(g_m[2,1],3,8)||shape(g_m[2,2],3,3)||shape(g_m[2,3],3,5)||shape(g_m[2,4],3,1))//
        	      (shape(g_m[3,1],5,8)||shape(g_m[3,2],5,3)||shape(g_m[3,3],5,5)||shape(g_m[3,4],5,1))//
	        	  (shape(g_m[4,1],1,8)||shape(g_m[4,2],1,3)||shape(g_m[4,3],1,5)||shape(g_m[4,4],1,1));


	gffpmp_m1 = (shape(g[1,1],3,3)||shape(g[1,2],3,4)||shape(g[1,3],3,1))//
         		(shape(g[2,1],4,3)||shape(g[2,2],4,4)||shape(g[2,3],4,1))//
         		(shape(g[3,1],1,3)||shape(g[3,2],1,4)||shape(g[3,3],1,1));
	gffpmp_m2 = (shape(g[4,4],2,2)||shape(g[4,5],2,1))//
      	        (shape(g[5,4],1,2)||shape(g[5,5],1,1));
	gffpmp_m3 = (shape(g[6,6],3,3)||shape(g[6,7],3,2))//
            	(shape(g[7,6],2,3)||shape(g[7,7],2,2));
	gffpmp_m4 = shape(g[8,8],1,1);
	gffpmp = 	block(gffpmp_m1,gffpmp_m2,gffpmp_m3,gffpmp_m4);


	bfm = shape(b_m1[1],3,17)//shape(b_m1[2],4,17)//shape(b_m1[3],1,17)//
      	 shape(b_m2[1],2,17)//shape(b_m2[2],1,17)//
	 	 shape(b_m3[1],3,17)//shape(b_m3[2],2,17)//
	 	 shape(b_m4[1],1,17);



	dm = block(j(8,8,1),j(3,3,1),j(5,5,1),j(1,1,1));
	dfm = block(j(3,3,1),j(4,4,1),j(1,1,1),j(2,2,1),j(1,1,1),j(3,3,1),j(2,2,1),j(1,1,1));
	marsh1 = (gamma - s0b)#(1/s`)#dfm;
	marsh2 = (s`*s0 + s0b)#(1/s`);
	marsh3 = gffpmp#(1/ufm) + u0fpmp;
	marsh4 = (bfm#u0fpmp#(1/ufm) + u0fpmp)#zcapmmp;
	marsh = marsh1 + marsh2#marsh3#dm + marsh2#marsh4 - i(17);


 	su =  (s[1:3]#u[1]#s_m[1])//(s[4:7]#u[2]#s_m[1])//(s[8]#u[3]#s_m[1])
			//(s[9:10]#u[4]#s_m[2])//(s[11]#u[5]#s_m[2])
       		//(s[12:14]#u[6]#s_m[3])//(s[15:16]#u[7]#s_m[3])
       		//(s[17]#u[8]#s_m[4]) ;

	su = su`;  /*su is now a row vector and  is the abs. share of a drug.*/

	sub = su@betaifm;
	hickadd =  (su`*su + sub)#(1/su`);

	hicks = marsh + marsh2#ufm;   /*this is what i sent to gary */
	hicks = marsh + hickadd;



  	m1f1names = {'Ritalin','Methylin','Generics (MPH-IR)'};
  	m1f2names = {'Ritalin SRLA', 'Metadate ERCD', 'MethylinER','Generics (MPH-ER)'};
  	m1f3names = {'Concerta'};
  	m2f1names = {'Aderall','Generic (MAS-IR)'};
  	m2f2names = {'Aderall XR'};
  	m3f1names = {'Dexederine','Dextrostat','Generics (DEX-IR)'};
 	 m3f2names = {'Dexederine SR','Generics (DEX-ER)'};
 	 m4f1names = {'Offlabels'};
  	names1 = m1f1names//m1f2names//m1f3names//m2f1names//m2f2names//m3f1names//m3f2names//m4f1names;

  	m1names = {'MPH-IR','MPH-ER','MPH-OROS'};
 	m2names = {'MAS-IR', 'MAS-ER'};
  	m3names = {'DEX-IR', 'DEX-ER'};


 	m1names = {'MPH-IR','MPH-ER','MPH-OROS'};
 	m2names = {'MAS-IR','MAS-ER'};
  	m3names = {'DEX-IR','DEX-ER'};
  	m4names = {'Off-Labels+ (5 Drugs)'};
  	names2 = m1names//m2names//m3names//m4names;

  	mnames = {'MPH','MAS','DEX','OFF-Lables +'};

	mattrib gamma label = {'gamma_ij Within Molecule M Form F'}
	colname = names1 rowname = names1;

	mattrib g label = {'g_ij Within Molecule M'}
	colname = names2 rowname = names2;

	mattrib gammacap label = {'GAMMA_mn All Molecules'}
	colname = mnames rowname = mnames;

	mattrib s label = {'s: Bottom Level Shares (Within Molecule-Form)'}
	format = 9.3
	colname = names1;

	mattrib u label = {'u: Mid-Level Shares (Share of Forms within Molecule)'}
	format = 9.3
	colname = names2;


	mattrib marsh label = {'Unconditional Marshallian Elasticities'}
	format = 9.3
	colname = names1 rowname = names1;

	mattrib hicks label = {'Matrix of Hicksian Elasticities'}
	format = 9.4
	colname = names1 rowname = names1;


	mattrib su label = {'Absolute Shares of Each Drug'}
	format = 9.3 colname = names1;


	mattrib s_m label = {'Share of Each Molecule'}
	format = 9.3 colname = mnames;


	print s_m;
	print u;
	print s;
	print su;
	print marsh;
	*print hicks;

    reset storage = &marshcat;
    store s su names1;
    close;

 %macro myprint(seg,segname);
 	 m = emij_&seg;
  	 h = ehij_&seg;
	 a = haij_&seg;
  	 e = etap_&seg;  e = e`;
  	 s = s_&seg;     s = s`;
	 k = &segname;
  	 kk = k//k//k//{'Exp','Share'};

  	 mat = m||h||a||e||s;

  	 mattrib mat label = {"Conditional Elastcities (Marshallian/Hicksian/H-Allen/Expendiure) & Shares"}
  	format = 9.2 rowname = k colname = kk;
  	print mat;
%mend;

%myprint(m1f1,m1f1names);
%myprint(m1f2,m1f2names);
%myprint(m1,m1names);
%myprint(m2f1,m2f1names);
%myprint(m2,m2names);
%myprint(m3f1,m3f1names);
%myprint(m3f2,m3f2names);
%myprint(m3,m3names);
%myprint(m,mnames);



topeq = b_top||g_top;
mattrib topeq label = {"Estimates of the Top Level Equation"}
colname = {"B", "Gamma"};
print topeq;


finish unconditionals;

print "-------------------- 3SLS RESULTS --------------------";
reset storage = work.oneiv;	 load;  /*read in 2SLS results for computing unconditional elasticities*/
reset storage = work.twoiv;  load;  /*read in 2SLS  based conditional elasticities for re-printinting*/
run unconditionals;
reset storage = &marshcat;
eiv = marsh;
store eiv;
close;

print "-------------------- SUR RESULTS ---------------------";
reset storage = work.oneols; load;	    /*read in OLS results for computing unconditional elasticities*/
reset storage = work.twools; load;      /*read in 2SLS  based conditional elasticities for re-printinting*/
run unconditionals;
reset storage = &marshcat;
eols = marsh;
store eols;
close;

run; quit;
ods html close;




proc datasets lib = work;
  save
  m1f1results m1f1resultsb m1f2results m1f2resultsb m1results m1resultsb
  m2f1results m2f1resultsb m2results m2resultsb
  m3f1results m3f1resultsb m3f2results m3f2resultsb m3results m3resultsb
  mresults mresultsb
  topresults &marshcat
  ;
run; quit;



/************************************* STEP 13: *******************************************

STEP 13:   Combine all the previous results (coefficents and variances) into fewer
           datasets and then print all regression results.
     

STEP13a:  (i) For any given segment, say M1F1, it combines the regression coeffients and
          covarainces into a single dataset (recall that estimates on coeffiencts related
          to prices and expenxditures were saved in onedata set while the coefficients
          on other exogenous variables were saved in another dataset 
          (example, m1f1results and m1f1resultsB are combined into m1f1resultsX).
           Insert the name of the segment (eg M1F1, M2F2 etc. into the combined
           dataset named yyyyyX.   


          (ii) When creating a new combined dataset, it retains only the estimated
           value of the parameters and the standard error of the estimate (ie discards
           covariances between parameters as it is not needed for printing results).

          (iii) IMPORTANT:  The original regressions could have been done by demeaning the
           data at state level (so as to mimic including state dummies). IF THIS WERE DONE
          then the estimatated covariances would be off because the canned routines do not
           know that the data has been demeaned.  This this code also correctes for those
           missing degrees of freedom in the SE.  
           This is done by setting the variable 
           STATE = 48
           in the code below. 
           IF HOWEVER the data has not been demeaned around states, then no correction is
           needed to get the correct SE (i.e. SAS is already computing the correct SEs). In
           this case set 
           STATE = 0.

           (iv) Finally, since these steps need to be perfomred on each of the earlier 
            set of regressions, do so by defining a macro 'PARMSE'.

STEP 13.b.  Combine all regression coefficients and se (from all regressions) into 
            one combined dataset 'WORK.MODELBETASE'. A variable named 'model' contains the 
            name of the segment, and a variable named 'type' tells what type of estimation
            gave these estimated (i.e. SUR,OLS,3SLS or 2SLS). 

STEP 13.c. Read in WORK.MODELBETASE into IML, put appropriate labels on results and print. 


***************************************************************************************/



/************************************* STEP 13.a ****************************************/
/*DEFINE MACRO PARMSE*/
%macro parmse(indata, outdata, mod);

data foo (drop = _status_);
  set &indata;
run; quit;


proc iml;
  use foo;
  read all var{_NAME_} into names where  (_TYPE_ = 'SUR' & _NAME_ ^= '');
  read all var _NUM_ into vsur where (_TYPE_ = 'SUR' & _NAME_ ^= '');
  read all var _NUM_ into viv where (_TYPE_ = '3SLS' & _NAME_ ^= '');
  read all var _NUM_ into bsur where (_TYPE_ = 'SUR' & _NAME_ = '');
  read all var _NUM_ into biv where (_TYPE_ = '3SLS' & _NAME_ = '');
  close foo;


  k = ncol(vsur);
  nt = vsur[1,1];
  /**********************************************************************************
    Set STATES = 48 if data was demeaned by states before regressions were estimated. 
    else set STATES = 0;
  *********************************************************************************/
  *** states = 48;     
  states = 0;

  vsur = vsur[,2:k] ;
  viv = viv[,2:k] ;
  bsur = bsur[,2:k];
  biv = biv[,2:k];
  k = ncol(vsur);

  sdsur = sqrt(vecdiag(vsur)*((nt - k)/(nt - states - k)));
  sdiv = sqrt(vecdiag(viv)*((nt - k)/(nt - states - k)));

  out = bsur//sdsur`//biv//sdiv`;
  rownames =  {"SUR - Beta" "SUR - SD" "3SLS - Beta" "3SLS - SD"};
  mattrib out colname = names
              rowname = rownames;
  *print out;

   create work.foo from out [colname = names];
   append from out;
   close work.foo;

   type = {'SUR' 'PARM', 'SUR' 'SE', '3SLS' 'PARM', '3SLS' 'SE'};
   create work.label from type[colname = {'_TYPE_' 'STAT'}];
   append from type;
   close work.label;


quit;

data &outdata;
  model = &mod;
  merge label foo;
run; quit;

proc datasets lib=work nolist;
delete label foo;
run; quit;
%mend;
/************************* END OF MACRO PARMSE **************************/


%parmse(m1f1results,m1f1resultsx,'MPH-IR');
%parmse(m1f1resultsb,m1f1resultsbx,'MPH-IR');
data m1f1resultsx ;  merge m1f1resultsx m1f1resultsbx; run; quit;

%parmse(m1f2results,m1f2resultsx, 'MPH-ER' );
%parmse(m1f2resultsb,m1f2resultsbx, 'MPH-ER');
data m1f2resultsx ;  merge m1f2resultsx m1f2resultsbx; run; quit;

%parmse(m1results,m1resultsx, 'MPH');
%parmse(m1resultsb,m1resultsbx, 'MPH');
data m1resultsx ;  merge m1resultsx m1resultsbx; run; quit;

%parmse(m2f1results,m2f1resultsx, 'MAS-IR');
%parmse(m2f1resultsb,m2f1resultsbx, 'MAS-IR');
data m2f1resultsx;  merge m2f1resultsx m2f1resultsbx; run; quit;


%parmse(m2results,m2resultsx, 'MAS');
%parmse(m2resultsb,m2resultsbx, 'MAS');
data m2resultsx;  merge m2resultsx m2resultsbx; run; quit;


%parmse(m3f1results,m3f1resultsx,'DEX-IR');
%parmse(m3f1resultsb,m3f1resultsbx,'DEX-IR');
data m3f1resultsx;  merge m3f1resultsx m3f1resultsbx; run; quit;

%parmse(m3f2results,m3f2resultsx,'DEX-ER');
%parmse(m3f2resultsb,m3f2resultsbx,'DEX-ER');
data m3f2resultsx;  merge m3f2resultsx m3f2resultsbx; run; quit;


%parmse(m3results,m3resultsx,'DEX');
%parmse(m3resultsb,m3resultsbx,'DEX');
data m3resultsx;  merge m3resultsx m3resultsbx; run; quit;


data mresultsx;
  set mresults;
  if _type_ = 'OLS' then _type_ = 'SUR';
  if _type_ = '2SLS' then _type_ = '3SLS';
 run; quit;


data mresultsbx;
  set mresultsb;
  if _type_ = 'OLS' then _type_ = 'SUR';
  if _type_ = '2SLS' then _type_ = '3SLS';
 run; quit;


%parmse(mresultsx,mresultsx,'MOL');
%parmse(mresultsbx,mresultsbx,'MOL');
data mresultsx;  merge mresultsx mresultsbx; run; quit;


data topresultsx;
  set topresults;
  if _type_ = 'OLS' then _type_ = 'SUR';
  if _type_ = '2SLS' then _type_ = '3SLS';
 run; quit;


%parmse(topresultsx,topresultsx,'TOP');


/************************************* STEP 13.b ****************************************/

data modelbetase (drop = a1);
   length model _type_ stat $8. ;
   length b1 b2 b3 b4
          g11 g12 g13 g14
          g21 g22 g23 g24
          g31 g32 g33 g34
          g41 g42 g43 g44
          c11 c12 c13 c14 c15 c16 c17
          c21 c22 c23 c24 c25 c26 c27
          c31 c32 c33 c34 c35 c36 c37
          c41 c42 c43 c44 c45 c46 c47  8.;

 set  m1f1resultsx m1f2resultsx m1resultsx
	  m2f1resultsx m2resultsx
	  m3f1resultsx m3f2resultsx	m3resultsx
      mresultsx topresultsx;
	  if model = 'MOL' and _TYPE_ = 'SUR' then _TYPE_ = 'OLS';
	  if model = 'MOL' and _TYPE_ = '3SLS' then _TYPE_ = '2SLS';
	  if model = 'TOP' and _TYPE_ = 'SUR' then _TYPE_ = 'OLS';
	  if model = 'TOP' and _TYPE_ = '3SLS' then _TYPE_ = '2SLS';

 rename _type_ = type;


run; quit;

proc datasets lib = work;
  save modelbetase &marshcat;
run; quit;



/************************************* STEP 13.c ****************************************/

ods html file = myfile2 newfile = none;

title1 'Regression coeffcients and SEs for Model Ver15l';
title2 'Created via SAS program aids-estimation-ver15l-2-analysis.sas';
title3 'Note: SE have been corrected for 48 df of state dummies';




proc iml;

/**************************** M1F1 Results (MPH-IR) ****************************/
 use modelbetase;
 read all where (model = 'MPH-IR' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//g13//c13//c14//c15//c16//c11//c12//c17;
  s2 = b2//g21//g22//g23//c23//c24//c25//c26//c21//c22//c27;
  s3 = b3//g31//g32//g33//c33//c34//c35//c36//c31//c32//c37;
  MPHIR = s1||s2||s3;

   colnames1 = {"s1 (Ritalin)" "s2 (Methylin)" "s3 (Generics)"};
   rownames1 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnp3" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" "" "timeCb" ""};
   rownames1 = rownames1`;

  mattrib mphir rowname = rownames1
            colname = colnames1
            label = "Segment MPH-IR (M1F1)"
		    format = 5.3;

  *print mphir;

/**************************** M1F2 Results (MPH-ER) ****************************/
 use modelbetase;
 read all where (model = 'MPH-ER' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//g13//g14//c13//c14//c15//c16//c11//c12//c17;
  s2 = b2//g21//g22//g23//g24//c23//c24//c25//c26//c21//c22//c27;
  s3 = b3//g31//g32//g33//g34//c33//c34//c35//c36//c31//c32//c37;
  s4 = b4//g41//g42//g43//g44//c43//c44//c45//c46//c41//c42//c47;
  mpher = s1||s2||s3||s4;

   colnames2 = {"s1 (Ritalin SR/LA" "s2 (Metadate ER/CD)" "s3 (Methylin ER)" "s4 (Generics)"};
   rownames2 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnp3" "" "lnp4" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" "" "timeCb" ""};
   rownames2 = rownames2`;

  mattrib mpher rowname = rownames2
            colname = colnames2
            label = "Segment MPH-ER (M1F2)"
            format = 5.3;

*  print mpher;

/**************************** M1 Results (MPH) ****************************/
 use modelbetase;
 read all where (model = 'MPH' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//g13//c13//c14//c15//c16//c11//c12;
  s2 = b2//g21//g22//g23//c23//c24//c25//c26//c21//c22;
  s3 = b3//g31//g32//g33//c33//c34//c35//c36//c31//c32;
  mph = s1||s2||s3;

   colnames3 = {"u1 (MPH-IR)" "u2 (MPH-ER)" "u3 (MPH-OROS (Concerta))"};
   rownames3 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnp3" ""  "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames3 = rownames3`;

  mattrib mph rowname = rownames3
            colname = colnames3
            label = "Share Equations for Segment MPH (M1)"
            format = 5.3;

*  print mph;

/**************************** M2F1 Results (MAS-IR) ****************************/
 use modelbetase;
 read all where (model = 'MAS-IR' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//c13//c14//c15//c16//c11//c12;
  s2 = b2//g21//g22//c23//c24//c25//c26//c21//c22;
  masir = s1||s2;

   colnames4 = {"s1 (Adderall)" "s2 (Generics)"} ;
   rownames4 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames4 = rownames4`;

  mattrib masir rowname = rownames4
            colname = colnames4
            label = "Share Equations for Segment MAS-IR (M2F1)"
            format = 5.3;

 * print masir;

/**************************** M2 Results (MAS) ****************************/

 use modelbetase;
 read all where (model = 'MAS' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//c13//c14//c15//c16//c11//c12;
  s2 = b2//g21//g22//c23//c24//c25//c26//c21//c22;
  mas = s1||s2;

   colnames5 = {"u1 (MAS-IR)" "u2 (MAS-ER (Adderall XR))"} ;
   rownames5 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames5 = rownames5`;

  mattrib mas rowname = rownames5
            colname = colnames5
            label = "Share Equations for Segment MAS (M2)"
            format = 5.3;


*  print mas;

/**************************** M3F1 Results (DEX-IR) ****************************/
 use modelbetase;
 read all where (model = 'DEX-IR' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//g13//c13//c14//c15//c16//c11//c12;
  s2 = b2//g21//g22//g23//c23//c24//c25//c26//c21//c22;
  s3 = b3//g31//g32//g33//c33//c34//c35//c36//c31//c32;
  dexir = s1||s2||s3;

   colnames6 = {"s1 (Dexedrine)" "s2 (Dextrostat)" "s3 (Generics)"};
   rownames6 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnp3" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames6 = rownames6`;

  mattrib dexir rowname = rownames6
            colname = colnames6
            label = "Share Equations for Segment DEX-IR (M3F1)"
            format = 5.3;

*  print dexir;

/**************************** M3F2 Results (DEX-ER) ****************************/
 use modelbetase;
 read all where (model = 'DEX-ER' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//c13//c14//c15//c16//c11//c12;
  s2 = b2//g21//g22//c23//c24//c25//c26//c21//c22;
  dexer = s1||s2;

   colnames7 = {"s1 (Dexedrine SR)" "s2 (Generics)"} ;
   rownames7 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames7 = rownames7`;

  mattrib dexer rowname = rownames7
            colname = colnames7
            label = "Share Equations for Segment DEX-ER (M3F2)"
            format = 5.3;

*  print dexer;

/**************************** M3 Results (DEX) ****************************/
 use modelbetase;
 read all where (model = 'DEX' & type = '3SLS');
 close modelbetase;


  s1 = b1//g11//g12//c13//c14//c15//c16//c11//c12;
  s2 = b2//g21//g22//c23//c24//c25//c26//c21//c22;
  dex = s1||s2;

   colnames8 = {"u1 (DEX-IR)" "s2 (DEX-ER)"} ;
   rownames8 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames8 = rownames8`;

  mattrib dex rowname = rownames8
            colname = colnames8
            label = "Share Equations for Segment DEX (M3)"
            format = 5.3;

*  print dex;

/**************************** M Results (MOL) ****************************/

 use modelbetase;
 read all where (model = 'MOL' & type = '2SLS');
 close modelbetase;


  lnq1b = b1//g11//g12//g13//g14//c13//c14//c15//c16//c11//c12//c17//c18//c19;
  lnq2b = b2//g21//g22//g23//g24//c23//c24//c25//c26//c21//c22//c27//c28//c29;
  lnq3b = b3//g31//g32//g33//g34//c33//c34//c35//c36//c31//c32//c37//c38//c39;
  lnq4b = b4//g41//g42//g43//g44//c43//c44//c45//c46//c41//c42//c47//c48//c49;


  mol = lnq1b||lnq2b||lnq3b||lnq4b;

   colnames9 = {"lnQ1 (MPH)" "lnQ2 (MAS)"  "lnQ3 (DEX)"  "lnQ4 (OTH)"} ;
   rownames9 = {"lnR" "" "lnp1" "" "lnp2" "" "lnp3" "" "lnp4" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" "" "timeCb" "" "Rel Share: MPH-XR" "" "Rel Share: MAS-XR" ""};
   rownames9 = rownames9`;

  mattrib mol rowname = rownames9
            colname = colnames9
            label = "Ln Qnty Equations for Molecules (M)"
            format = 5.3;

 * print mol;

/**************************** TOP Results (TOP) ****************************/

 use modelbetase;
 read all where (model = 'TOP' & type = '2SLS');
 close modelbetase;

  top = b1//g11//c13//c14//c15//c16//c11//c12;

   colnames10 = {"lnQ (All Drugs)"} ;
   rownames10 = {"lnY" "" "lnP" "" "lnkids" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" ""};
   rownames10 = rownames10`;

  mattrib top rowname = rownames10
            colname = colnames10
            label = "Ln Qnty Equation for All Drugs"
            format = 5.3;

*  print top;

/****************************  PRINT ALL RESULTS *****************************/

print mphir;
print mpher;
print mph;
print masir;
print mas;
print dexir;
print dexer;
print dex;
print mol;
print top;

quit;

ods html close;
